class Admin::ArchiveFilesController < OrbitAdminController
  require 'action_dispatch/http/upload'
  def show
    module_pages = Page.where(:module => 'archive').collect{|p| p.url}
    if module_pages.length<1
        render :text => t('archive.no_page').to_s
    else
        redirect_to '/' + I18n.locale.to_s + module_pages[0]  + '?title=' + params['title'].to_s
    end
  end
  def save_categories_order
    keys = params['data_keys']
    values = params['data_values']
    keys.each do |key|
        key.slice! 'sort_number-'
    end
    keys.each_with_index do |key,index|
        archivecategory = ArchiveCategory.where(category_id: key)
        if !(values[index].empty?)
            archivecategory.first.update_attributes(sort_number: values[index].to_i)
        else
            archivecategory.first.update_attributes(sort_number: nil)
        end
    end
    render :json => {}.to_json
  end
  def index
    
    if ArchiveSortOrder.count == 0
        ArchiveSortOrder.create('sort_order' => false)
    end
    if !(params['order_asc'].nil?)
        if params['order_asc'] == 'true'
            ArchiveSortOrder.first.update_attributes('sort_order' => true)
        elsif params['order_asc'] == 'false'
            ArchiveSortOrder.first.update_attributes('sort_order' => false)
        end
    end
    @choose = ArchiveSortOrder.first['sort_order']
    @table_fields = [:status, :category, :title, :updated_at,:last_modified,"archive.downloaded_times"]
    @categories = @module_app.categories.enabled
    @tags = @module_app.tags
    @filter_fields = filter_fields(@categories, @tags)
    @archives = ArchiveFile.local_data.order_by(sort)
                                              .with_categories(filters("category"))
                                              .with_tags(filters("tag"))
                                              .with_status(filters("status"))

    @archives = search_data(@archives,[:title]).page(params[:page]).per(10)

    render :partial => "index" if request.xhr?
  end

  def new
    sort_number = ArchiveSortOrder.first.get_default_order
    @archive_file = ArchiveFile.new(:sort_number=>sort_number)
    @tags = @module_app.tags
    @categories = @module_app.categories.enabled
  end

  def edit
    @archive_file = ArchiveFile.find(params[:id])
    if can_edit_or_delete?(@archive_file)
     @tags = @module_app.tags
     @categories = @module_app.categories.enabled
    else
      render_401
    end
  end
  def categories_order
    categories = @module_app.categories
    cat_id_cat_maps = categories.collect{|cat| [cat.id.to_s, cat]}.to_h
    cat_id_sort_number_maps = ArchiveCategory.all.order_by(sort_number: 'desc').pluck(:category_id, :sort_number).to_h
    no_archive_cat_ids = cat_id_cat_maps.keys - cat_id_sort_number_maps.keys
    no_archive_cat_ids.each do |cat_id|
      ArchiveCategory.create(category_id: cat_id)
      cat_id_sort_number_maps[cat_id] = nil
    end
    @cats = cat_id_sort_number_maps.collect do |cat_id_sort_number_entry|
      cat_id = cat_id_sort_number_entry[0]
      sort_number = cat_id_sort_number_entry[1]
      [cat_id_cat_maps[cat_id], sort_number.to_s]
    end
    render :partial => "categories_order"  if request.xhr?
  end
  def create
    params["archive_file"]["archive_file_multiples_attributes"].each do |k,v|
      if v["file"].class == String
        file_content_info = JSON.parse(v["file"])
        content_type = file_content_info["type"]
        filename = file_content_info["name"]
        head = "Content-Disposition: form-data; name=\"archive_file[archive_file_multiples_attributes][#{k}][file]\"; filename=\"#{filename}\" Content-Type: #{content_type}" 
        tempfile = Tempfile.new(filename,binmode: true)
        tempfile.write(file_content_info["content"].unpack('U*').map{|i| i.chr }.join)
        v["file"] = ActionDispatch::Http::UploadedFile.new({:filename=>filename,:type=>content_type,:head=>head,:tempfile=>tempfile})
      end
    end rescue nil
    @archive_file = ArchiveFile.new(archive_vars)
    @archive_file.create_user_id = current_user.id
    @archive_file.update_user_id = current_user.id

    respond_to do |format|
      if @archive_file.save
        Thread.new do
          @archive_file.notify_feed("create")
        end
        format.html { redirect_to(admin_archive_files_path) }
        format.xml  { render :xml => @archive_file, :status => :created, :location => @archive_file }
      else
        @tags = @module_app.tags
        format.html { render :action => "new" }
        format.xml  { render :xml => @archive_file.errors, :status => :unprocessable_entity }
      end
    end
  end

  # PUT /archive_files/1
  # PUT /archive_files/1.xml
  def update
    params["archive_file"]["archive_file_multiples_attributes"].each do |k,v|
      if v["file"].class == String
        file_content_info = JSON.parse(v["file"])
        content_type = file_content_info["type"]
        filename = file_content_info["name"]
        head = "Content-Disposition: form-data; name=\"archive_file[archive_file_multiples_attributes][#{k}][file]\"; filename=\"#{filename}\" Content-Type: #{content_type}" 
        tempfile = Tempfile.new(filename,binmode: true)
        tempfile.write(file_content_info["content"].unpack('U*').map{|i| i.chr }.join)
        v["file"] = ActionDispatch::Http::UploadedFile.new({:filename=>filename,:type=>content_type,:head=>head,:tempfile=>tempfile})
      end
    end rescue nil
    @archive_file = ArchiveFile.find(params[:id])
  
    @archive_file.update_user_id = current_user.id

    respond_to do |format|
      if @archive_file.update_attributes(archive_vars)
        Thread.new do
          @archive_file.notify_feed("update")
        end
        format.html { redirect_to(admin_archive_files_path) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @archive_file.errors, :status => :unprocessable_entity }
      end
    end
  end

  def destroy
    archive_file = ArchiveFile.find(params[:id])
    archive_file.destroy
    Thread.new do
      archive_file.notify_feed("destroy")
    end
    redirect_to admin_archive_files_path
  end
  def delete
    if params[:ids]
      ArchiveFile.any_in(:uid => params[:ids]).destroy_all
      Thread.new do
        ArchiveFile.notify_feed_delete(params[:ids])
      end
    end
    if request.xhr?
      render :nothing => true, :status => 204
    else
      redirect_to admin_archive_files_path
    end
  end
  def feed
    @table_feed_fields = ["archive.feed_name",:tags , :category , "archive.rssfeed", "archive.jsonfeed"]
    @feeds = ArchiveFileFeed.all.asc(:created_at)  
  end
  def feedform
    if params[:type] == "new"
      @archive_feed = ArchiveFileFeed.new(id: nil)
      render :partial => "feed_form"
    else params[:type] == "edit"
      @archive_feed = ArchiveFileFeed.find(params[:id])
      render :partial => "edit_feed_form"
    end
  end

  def createfeed
    archive_feed = ArchiveFileFeed.new(feed_params)
    archive_feed.save
    #ArchiveFileFeed.create_feed_cache(nil,archive_feed)
    feeds = ArchiveFileFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end

  def updatefeed
    archive_feed  = ArchiveFileFeed.find(params[:id])
    archive_feed.update_attributes(feed_params)
    archive_feed.save
    #ArchiveFileFeed.create_feed_cache(nil,archive_feed)
    feeds = ArchiveFileFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end

  def deletefeed
    archive_feed  = ArchiveFileFeed.find(params[:id])
    archive_feed.destroy
    feeds = ArchiveFileFeed.all.asc(:created_at)
    render :partial => "feed", :collection => feeds
  end
  private
  def feed_params
    feed_params = params.require(:archive_file_feed).permit!
    if feed_params[:tag_ids].nil?
      feed_params[:tag_ids] = []
    end
    if feed_params[:category_ids].nil?
      feed_params[:category_ids] = []
    end
    feed_params
  end
  def archive_vars
    params[:archive_file][:tags] ||=[]
    params.require(:archive_file).permit!
  end

  def setup_vars
    @module_app = ModuleApp.where(:key => "archive").first
  end
    # def sort
    #   unless params[:sort].blank?
    #     case params[:sort]
    #      when "downloaded_times"
            
    #      else
    #       @sort = sort
    #     end
    #   else
    #     @sort = sort
    #   end
    #   @sort
    # end
end
